﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.ConditionActionEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.VariableVector {
  [StorableClass]
  [Item("VariableVectorInput", "")]
  public class VariableVectorInput : Dictionary<string, string>, IInput {

    public IOrderedEnumerable<string> Order { get { return this.Keys.OrderBy(x => x); } }

    [StorableConstructor]
    protected VariableVectorInput(bool deserializing) { }
    protected VariableVectorInput(VariableVectorInput original, Cloner cloner) {
      foreach (var item in original) {
        Add((string)item.Key.Clone(), (string)item.Value.Clone());
      }
    }
    public VariableVectorInput() : base() { }
    public VariableVectorInput(int capacity) : base(capacity) { }
    public VariableVectorInput(IDictionary<string, string> dictionary) : base(dictionary) { }

    public override string ToString() {
      StringBuilder sb = new StringBuilder();
      bool first = true;
      foreach (var item in this) {
        if (first) {
          first = false;
        } else {
          sb.Append("; ");
        }
        sb.Append(String.Format("{0}: {1}", item.Key, item.Value));
      }
      return sb.ToString();
    }

    public virtual string ItemName {
      get { return ItemAttribute.GetName(this.GetType()); }
    }
    public virtual string ItemDescription {
      get { return ItemAttribute.GetDescription(this.GetType()); }
    }
    public Version ItemVersion {
      get { return ItemAttribute.GetVersion(this.GetType()); }
    }
    public static Image StaticItemImage {
      get { return HeuristicLab.Common.Resources.VSImageLibrary.Class; }
    }
    public virtual Image ItemImage {
      get { return ItemAttribute.GetImage(this.GetType()); }
    }

    public virtual IDeepCloneable Clone(Cloner cloner) {
      return new VariableVectorInput(this, cloner);
    }

    public object Clone() {
      return Clone(new Cloner());
    }

    public event EventHandler ItemImageChanged;
    protected virtual void OnItemImageChanged() {
      EventHandler handler = ItemImageChanged;
      if (handler != null) handler(this, EventArgs.Empty);
    }
    public event EventHandler ToStringChanged;
    protected virtual void OnToStringChanged() {
      EventHandler handler = ToStringChanged;
      if (handler != null) handler(this, EventArgs.Empty);
    }
  }
}
